<?php
// $Id:
define('TRIGGER_TEMPLATES_PATH', drupal_get_path('module', 'code_gen') .'/templates/trigger');
function code_gen_trigger_module() {
  $content = '<div id="code_gen_create_trigger_form">';
    $content .= drupal_get_form('code_gen_create_trigger_form');
  $content .= '</div>';
  return $content;
}

function code_gen_create_trigger_form(&$form_state) {
  $form = array();
  $form['module_name'] = array(
    "#type" => "textfield",
    "#title" => t("Module name"),
    "#description" => t("Name of module to auto generate."),
    '#required' => TRUE,
  );
  $form['action_name'] = array(
    "#type" => "textfield",
    "#title" => t("Action name"),
    "#description" => t("Name of action to auto generate."),
    '#required' => TRUE,
  );
  $type_options = array(
    "system" => "System",
    "node" => "Node",
    "user" => "User",
    "comment" => "Comment",
    "taxonomy" => "Taxonomy",
  );
  $form['action_type'] = array(
    "#type" => "select",
    "#multiple" => FALSE,
    "#title" => t("Action type"),
    "#options" => $type_options,
    "#description" => t("The type of action to create (what object does your action work with)."),
    "#default_value" => "system",
    '#required' => TRUE,
  );
  $trigger_hooks = _code_gen_get_trigger_hooks();
  $form['trigger_hooks'] = array(
    "#type" => "select",
    "#multiple" => TRUE,
    "#title" => t("Trigger hooks"),
    "#options" => $trigger_hooks,
    "#description" => t("Enable action for the following triggers."),
    '#required' => TRUE,
  );
  $form['is_configurable'] = array(
    '#type' => 'radios',
    '#options' => array(0 => t('No'), 1 => t('Yes')),
    '#title' => t("Is action configurable?"),
    '#default_value' => 0,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create'),
  );

  return $form;
}

function code_gen_create_trigger_form_validate($form, &$form_state) {
  $do_not_create_yet = file_directory_path()."/".$form_state['values']['module_name'];
  if (file_check_directory($do_not_create_yet, !FILE_CREATE_DIRECTORY)) {
    form_set_error('path ', t('Directory already exists. System must be able to create new path at #full_path', array('#full_path' => $do_not_create_yet)));
  }
}

function code_gen_create_trigger_form_submit($form, &$form_state) {
  require_once drupal_get_path('module', 'code_gen') . '/code_gen.common.inc';
  $template_path = TRIGGER_TEMPLATES_PATH;

  $module_name = $form_state['values']['module_name'];
  $action_name = $form_state['values']['action_name'];
  $action_type = $form_state['values']['action_type'];
  $trigger_hooks = _code_gen_pass_trigger_hooks($form_state['values']['trigger_hooks']);
  $path = file_directory_path()."/".$module_name;

  _code_gen_create_standard_module_files($module_name, $path);
  //modulename.info
  _code_gen_copy_templates($template_path, "info.template",$path, $module_name.".info", array("%module_name" => $module_name));
  //admin.inc
  _code_gen_copy_templates($template_path, "admin.inc.template",$path, $module_name.".admin.inc", array("%module_name" => $module_name));
  //Conditional add of action configuration
  $action_is_configurable = "FALSE";
  $action_configuration = "";
  if($form_state['values']['is_configurable'] == 1) {
    $action_is_configurable = "TRUE";
    $action_configuration = strtr(file_get_contents(TRIGGER_TEMPLATES_PATH."/config.form.template"),
                            array("%module_name" => $module_name,
                                  "%action_name" => $action_name,
                                  ));
  }

  //modulename.module
  _code_gen_copy_templates($template_path, "module.template",$path, $module_name.".module",
                           array("%module_name" => $module_name,
                                 "%action_name" => $action_name,
                                 "%action_type" => $action_type,
                                 "%hooks" => $trigger_hooks,
                                 "%action_is_configurable" => $action_is_configurable,
                                 "%action_configuration" => $action_configuration,
                                 ));
}

function _code_gen_pass_trigger_hooks($trigger_hook_array) {
  $temp_result = array();
  foreach ($trigger_hook_array as $hook) {
    $exploaded_hook = explode (" : ", $hook);
    if(!isset($temp_result[$exploaded_hook[0]])) {
      $temp_result[$exploaded_hook[0]] = array();
    }
    $temp_result[$exploaded_hook[0]][] = "'".$exploaded_hook[1]."'";
  }
  $result = "";
  foreach ($temp_result as $hook_type => $ops) {
    $result .= "        '".$hook_type."' => array(".implode(", ",$ops)."),\n";
  }
  return $result;
}

/**
 * Get the list of trigger names.
 */
function _code_gen_get_trigger_hooks( ) {
  $result = array();
  $hooks = module_invoke_all('hook_info');
  foreach ($hooks as $module => $hook) {
    foreach ($hook as $hook_type => $operation) {
      foreach ($operation as $operation_key => $operation_runs_when) {
        $result[$hook_type." : ".$operation_key] = $hook_type." : ".$operation_key;
      }
    }
  }
  return $result;
}
