<?php
// $Id:
define('VIEWS_TEMPLATES_PATH', drupal_get_path('module', 'code_gen') .'/templates/views');
$field_defs = array();

function code_gen_views_definitions_module() {
  $content = '<div id="code_gen_create_views_definitions_form">';
    $content .= drupal_get_form('code_gen_create_views_definitions_form');
  $content .= '</div>';
  return $content;
}

function code_gen_create_views_definitions_form(&$form_state) {
  $form = array();
  $form['module_name'] = array(
    "#type" => "textfield",
    "#title" => t("Module name"),
    "#description" => t("Name of module to auto generate."),
    '#required' => TRUE,
  );
  $tables = _code_gen_get_table_names();
  $form['code_gen_tables'] = array(
    "#type" => "select",
    "#multiple" => TRUE,
    "#title" => t("Create view definitions for the following tables"),
    "#options" => $tables,
    "#description" => t("The selected tables will be analyzed and views definitions will be made."),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create'),
  );
  return $form;
}
function code_gen_create_views_definitions_form_validate($form, &$form_state) {
  $do_not_create_yet = file_directory_path()."/".$form_state['values']['module_name'];
  if (file_check_directory($do_not_create_yet, !FILE_CREATE_DIRECTORY)) {
    form_set_error('path ', t('Directory already exists. System must be able to create new path at #full_path', array('#full_path' => $do_not_create_yet)));
  }
}

function code_gen_create_views_definitions_form_submit($form, &$form_state) {
  global $field_defs;
  global $join_defs;
  global $db_prefix;
  require_once drupal_get_path('module', 'code_gen') . '/code_gen.common.inc';

  $template_path = VIEWS_TEMPLATES_PATH;
  file_scan_directory($template_path, '^field.[A-Za-z]+.template$', array('.','..','CVS'), '_code_gen_pass_fields', FALSE);
  file_scan_directory($template_path, '^join.[A-Za-z]+.template$', array('.','..','CVS'), '_code_gen_pass_joins', FALSE);

  $module_name = $form_state['values']['module_name'];
  $path = file_directory_path()."/".$module_name;

  _code_gen_create_standard_module_files($module_name, $path);
  //modulename.info
  _code_gen_copy_templates($template_path, "info.template",$path, $module_name.".info", array("%module_name" => $module_name, "%table_names" => implode(", ",$form_state['values']['code_gen_tables'])));
  //modulename.module
  _code_gen_copy_templates($template_path, "module.template",$path, $module_name.".module", array("%module_name" => $module_name));
  //Write the views.inc file

  // table prefix determination according to db_prefix_tables()
  if (is_array($db_prefix)) {
    $prefix = FALSE;
    foreach ($db_prefix as $key => $val) {
      if (0 < ($pos = strpos($table_name, $key) && $val == ($tmp_prefix = substr($table_name, 0, $pos)) && (strlen($table_name) == strlen($key) + strlen($tmp_prefix) ))) {
        $prefix = $tmp_prefix;
      }
    }
    if ($prefix === FALSE){
      if (array_key_exists('default', $db_prefix)) {
        $prefix = $db_prefix['default'];
      } else {
        $prefix = '';
      }
    }
  }
  else {
    $prefix = $db_prefix;
  }
  $result = "";
  foreach ($form_state['values']['code_gen_tables'] as $table_name) {
    $result .= " // Definitions for ".$table_name."\n";
    $result .= "  \$data['".$table_name."']['table']['group'] = t('".strtr($table_name, array("_" => " "))."');\n";

    // prefix elimination from table name
    if ($prefix) {
      $table_name = strtr($table_name, array( $prefix => '' ));
    }

    $schema = drupal_get_schema($table_name);
    if ($schema) {
      $result .= "  \$".$table_name." = drupal_get_schema('".$table_name."');\n";
      foreach ($schema['fields'] as $field_name => $field_values) {
        $result .= "  //".$field_values['type']."\n";
        $field_str = $field_defs[$field_values['type']];
        $result .= strtr($field_str, array("%table_name" => $table_name, "%field_name" => $field_name));
        if ($join_defs[$field_name]) {
          $result .= strtr($join_defs[$field_name], array("%table_name" => $table_name, "%field_name" => $field_name));
        }
      }
    } else {
      //OOps - this must be a db view rather than a table!
      //Use describe in stead of drupal_get_schema - this will work for tables too but we will miss the posibility
      // Of using the column description as field help
      $fields = db_query("describe {".$table_name."}");
      while ( $field_values = db_fetch_array($fields) ) {
        $field_name = $field_values['Field'];
        $field_type = substr($field_values['Type'], 0, strlen($field_values['Type']) - strlen(strstr($field_values['Type'], "(")));
        $result .= "  //".$field_type."\n";
        $field_str = strtr($field_defs[$field_type], array("\$%table_name['fields']['%field_name']['description']" => "'".$table_name." : ".$field_name."'"));
        $result .= strtr($field_str, array("%table_name" => $table_name, "%field_name" => $field_name));
        if ($join_defs[$field_name]) {
          $result .= strtr($join_defs[$field_name], array("%table_name" => $table_name, "%field_name" => $field_name));
        }
      }
    }


  }
  $path .= "/"."views";
  if (!file_check_directory($path, FILE_CREATE_DIRECTORY)) {
    form_set_error('path', '"'.$path.t('" is not a valid file path. System must be able to write to #full_path', array('#full_path' => $path)));
    return;
  }
  _code_gen_copy_templates($template_path, "views.inc.template",$path, $module_name.".views.inc", array("%module_name" => $module_name, "%views_definitions" => $result));
}

/**
 * Get the list of table names.
 */
function _code_gen_get_table_names( ) {
  $out = "";
  // get auto_increment values and names of all tables
  $tables = db_query("show table status");
  while ( $table = db_fetch_array($tables) ) {
      $out[$table['Name']] = $table['Name'];
  }
  return $out;
}

/**
 * Pass templates for known db field types
 */
function _code_gen_pass_fields($file_name) {
  global $field_defs;
  $db_type = substr($file_name,
                    strlen(VIEWS_TEMPLATES_PATH."/field."),
                    strlen($file_name) - strlen(VIEWS_TEMPLATES_PATH."/field.") - strlen(".template"));
  $field_defs[$db_type] = file_get_contents($file_name);
}

/**
 * Pass templates for known join types
 */
function _code_gen_pass_joins($file_name) {
  global $join_defs;
  $field_name = substr($file_name,
                       strlen(VIEWS_TEMPLATES_PATH."/join."),
                       strlen($file_name) - strlen(VIEWS_TEMPLATES_PATH."/join.") - strlen(".template"));
  $join_defs[$field_name] = file_get_contents($file_name);
}
